# !usr/bin/env python
# -*- coding: utf-8 -*-
#
# Licensed under a 3-clause BSD license.
#
# @Author: Brian Cherinka
# @Date:   2017-11-29 10:28:58
# @Last modified by:   Brian Cherinka
# @Last Modified time: 2018-07-27 17:22:02

from __future__ import print_function, division, absolute_import
import pytest
from tree.tree import Tree
import six
import os


@pytest.fixture()
def tree():
    t = Tree()
    yield t
    t = None


class TestTree(object):

    def test_load(self, tree):
        assert tree.treedir is not None
        assert isinstance(tree.treedir, six.string_types)
        assert tree.environ is not None
        assert 'APO' in tree.environ
        assert 'sdsswork' == tree.environ['default']['name']

    @pytest.mark.parametrize('keys', [('manga'), ('MANGA'), (['manga', 'apogee'])],
                             ids=['manga', 'capmanga', 'apomanga'])
    def test_load_keys(self, keys):
        t = Tree(key=keys)
        assert 'APO' not in t.environ
        assert 'default' in t.environ
        assert 'general' in t.environ
        keys = keys if isinstance(keys, list) else [keys]
        for key in keys:
            assert key.upper() in t.environ

    @pytest.mark.parametrize('dr', [('dr15')])
    def test_load_dr_config(self, dr):
        t = Tree(config=dr)
        assert dr == t.environ['default']['name']

    def _assert_paths(self, tree, dr=None):
        assert 'manga_spectro_redux' in tree.environ['MANGA']
        assert 'manga_spectro_analysis' in tree.environ['MANGA']
        if dr:
            assert dr in tree.environ['default']['name']
            assert dr in tree.environ['MANGA']['manga_root']
            assert dr in tree.environ['MANGA']['manga_spectro_redux']
            assert dr in tree.environ['MANGA']['manga_spectro_analysis']

    @pytest.mark.parametrize('dr', [('dr15')])
    def test_load_with_update(self, tree, dr):
        assert 'sdsswork' in tree.environ['default']['name']
        assert 'mangawork' in tree.environ['MANGA']['manga_root']
        tree = Tree(config=dr, update=True)
        self._assert_paths(tree, dr=dr)

    @pytest.mark.parametrize('dr', [('dr15')])
    def test_replant_tree(self, tree, dr):
        assert 'sdsswork' in tree.environ['default']['name']
        assert 'mangawork' in tree.environ['MANGA']['manga_root']
        tree.replant_tree(dr)
        self._assert_paths(tree, dr=dr)

    def test_custom_paths_remain(self, monkeypatch):
        # set custom environ variable
        name = 'MANGA_QUICK'
        path = '/tmp/work/manga/quick'
        monkeypatch.setitem(os.environ, name, path)
        assert os.environ[name] == path
        tree = Tree()
        assert os.environ[name] == path
        assert path != tree.environ['MANGA'][name.lower()]

    def test_custom_paths_donotupdate(self, monkeypatch, tree):
        name = 'MANGA_QUICK'
        path = '/tmp/work/manga/quick'
        monkeypatch.setitem(os.environ, name, path)
        assert os.environ[name] == path
        tree.replant_tree('sdsswork')
        assert os.environ[name] != path
        assert os.environ[name] == tree.environ['MANGA'][name.lower()]

    def test_custom_paths_update_with_exclude(self, monkeypatch, tree):
        name = 'MANGA_QUICK'
        path = '/tmp/work/manga/quick'
        monkeypatch.setitem(os.environ, name, path)
        assert os.environ[name] == path
        tree.replant_tree('sdsswork', exclude=['MANGA_QUICK'])
        assert os.environ[name] == path
        assert path != tree.environ['MANGA'][name.lower()]






#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# @Author: Brian Cherinka, José Sánchez-Gallego, and Brett Andrews
# @Date: 2016-06-22
# @Filename: test_maps.py
# @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
#
# @Last modified by: José Sánchez-Gallego (gallegoj@uw.edu)
# @Last modified time: 2018-11-09 10:00:49


from __future__ import absolute_import, division, print_function

import copy
import re
from os.path import join

import astropy
import astropy.io.fits
import numpy as np
import pytest

import marvin
import marvin.tools.spaxel
from marvin.core.exceptions import MarvinError
from marvin.tests import marvin_test_if
from marvin.tools.maps import Maps
from marvin.utils.datamodel.dap.base import Property


def _assert_maps(maps, galaxy):
    """Basic checks for a Maps object."""

    assert maps is not None
    assert maps.plateifu == galaxy.plateifu
    assert maps.mangaid == galaxy.mangaid
    assert maps.wcs is not None
    assert maps.bintype == galaxy.bintype

    assert len(maps._shape) == len(galaxy.shape)
    for ii in range(len(maps._shape)):
        assert maps._shape[ii] == galaxy.shape[ii]


class TestMaps(object):

    def _get_maps_kwargs(self, galaxy, data_origin):

        if data_origin == 'file':
            maps_kwargs = dict(filename=galaxy.mapspath)
        else:
            maps_kwargs = dict(plateifu=galaxy.plateifu, release=galaxy.release,
                               bintype=galaxy.bintype, template=galaxy.template,
                               mode='local' if data_origin == 'db' else 'remote')

        return maps_kwargs

    def test_load(self, galaxy, exporigin):

        maps = Maps(**self._get_maps_kwargs(galaxy, exporigin))

        _assert_maps(maps, galaxy)

        assert maps.data_origin == exporigin

        if exporigin == 'file':
            assert isinstance(maps.data, astropy.io.fits.HDUList)
        elif exporigin == 'db':
            assert isinstance(maps.data, marvin.marvindb.dapdb.File)

        cube = maps.getCube()

        assert cube is not None
        assert cube.plateifu == galaxy.plateifu
        assert cube.mangaid == galaxy.mangaid

    @pytest.mark.parametrize('objtype, errmsg',
                             [('cube', 'Trying to open a non DAP file with Marvin Maps'),
                              ('models', 'Trying to open a DAP LOGCUBE with Marvin Maps')])
    def test_maps_wrong_file(self, galaxy, objtype, errmsg):
        path = galaxy.cubepath if objtype == 'cube' else galaxy.modelpath
        with pytest.raises(MarvinError) as cm:
            Maps(filename=path)
        assert errmsg in str(cm.value)

    def test_nobintype_in_db(self, galaxy):

        if galaxy.release != 'MPL-6':
            pytest.skip('only running this test for MPL6')

        with pytest.raises(MarvinError) as cm:
            maps = Maps(plateifu=galaxy.plateifu, bintype='ALL', release=galaxy.release)

        assert 'Specified bintype ALL is not available in the DB' in str(cm.value)

    @pytest.mark.slow
    def test_datamodel(self, galaxy, exporigin):

        maps = Maps(**self._get_maps_kwargs(galaxy, exporigin))

        fin = 'manga-{0}-{1}-MAPS-{2}.fits.gz'.format(galaxy.plate, galaxy.ifu, galaxy.bintemp)
        path = join(galaxy.mangaanalysis, galaxy.drpver, galaxy.dapver, galaxy.bintemp,
                    str(galaxy.plate), galaxy.ifu, fin)
        hdus = astropy.io.fits.open(path)

        for hdu in hdus[1:]:

            if ('IVAR' in hdu.name) or ('MASK' in hdu.name) or ('SIGMACORR' in hdu.name):
                continue

            name = hdu.name.lower()
            data = hdu.data
            header = hdu.header

            if len(data.shape) < 3:
                val = maps.getMap(name, exact=True).value
                assert val == pytest.approx(data, 0.0001), name

            else:
                for kk, vv in header.items():
                    channel_num = re.match('^[0-9]+$', kk[1:])

                    if (kk[0] == 'C') and (channel_num is not None):
                        channel = '_'.join(re.findall(r"[\w']+", vv)).lower()
                        fullname = '_'.join((name, channel))
                        val = maps.getMap(fullname, exact=True).value
                        assert val == pytest.approx(data[int(kk[1:]) - 1], 0.0001), name

    @pytest.mark.parametrize('monkeyconfig', [('release', 'MPL-5')], indirect=True)
    def test_load_mpl4_global_mpl5(self, galaxy, monkeyconfig, data_origin):

        assert marvin.config.release == 'MPL-5'
        maps = Maps(**self._get_maps_kwargs(galaxy, data_origin))

        assert maps.release == galaxy.release
        assert maps._drpver == galaxy.drpver
        assert maps._dapver == galaxy.dapver

    def test_maps_redshift(self, maps, galaxy):
        redshift = maps.nsa.redshift \
            if maps.release == 'MPL-4' and maps.data_origin == 'file' else maps.nsa.z
        assert redshift == pytest.approx(galaxy.redshift)

    def test_release(self, galaxy):
        maps = Maps(plateifu=galaxy.plateifu)
        assert maps.release == galaxy.release

    def test_set_release_fails(self, galaxy):
        maps = Maps(plateifu=galaxy.plateifu)
        with pytest.raises(MarvinError) as ee:
            maps.release = 'a'
            assert 'the release cannot be changed' in str(ee.exception)

    def test_deepcopy(self, galaxy):
        maps1 = Maps(plateifu=galaxy.plateifu)
        maps2 = copy.deepcopy(maps1)

        for attr in vars(maps1):
            if not attr.startswith('_'):
                value = getattr(maps1, attr)
                value2 = getattr(maps2, attr)

                if isinstance(value, np.ndarray):
                    assert np.isclose(value, value2).all()

                elif isinstance(value, astropy.wcs.wcs.WCS):
                    for key in vars(value):
                        assert getattr(value, key) == getattr(value2, key)

                elif isinstance(value, marvin.tools.cube.Cube):
                    pass

                elif attr in ['vacs']:
                    pass

                elif (isinstance(value, list) and len(value) > 0 and
                      isinstance(value[0], Property)):
                        for property1, property2 in zip(value, value2):
                            assert property1 == property2

                else:
                    assert value == value2, attr

    def test_getMapRatio(self, galaxy):
        maps = Maps(galaxy.plateifu)
        map_ratio = maps.getMapRatio('emline_gflux', 'nii_6585', 'ha_6564')
        map_arith = maps.emline_gflux_nii_6585 / maps.emline_gflux_ha_6564

        assert map_ratio.value == pytest.approx(map_arith.value, nan_ok=True)
        assert map_ratio.ivar == pytest.approx(map_arith.ivar, nan_ok=True)
        assert map_ratio.mask == pytest.approx(map_arith.mask, nan_ok=True)


class TestMaskbit(object):

    @marvin_test_if(mark='include', maps_release_only=dict(release=['MPL-4']))
    def test_quality_flag_mpl4(self, maps_release_only):
        ha = maps_release_only['emline_gflux_ha_6564']
        assert ha.quality_flag is None

    @pytest.mark.parametrize('flag',
                             ['manga_target1',
                              'manga_target2',
                              'manga_target3',
                              'target_flags'])
    def test_flag(self, flag, maps_release_only):
        ha = maps_release_only['emline_gflux_ha_6564']
        assert getattr(ha, flag, None) is not None



# !usr/bin/env python
# -*- coding: utf-8 -*-
#
# Licensed under a 3-clause BSD license.
#
# @Author: Brian Cherinka
# @Date:   2018-07-12 15:18:38
# @Last modified by:   Brian Cherinka
# @Last Modified time: 2018-07-12 15:31:26

from __future__ import print_function, division, absolute_import
from marvin.web.web_utils import buildImageDict
from marvin.utils.general.images import getImagesByPlate


class TestBuildImageDict(object):

    def test_placeholder(self):
        images = buildImageDict(None, test=True, num=5)
        assert len(images) == 5
        assert images[0]['name'] == '4444-0000'
        assert 'placehold.it' in images[0]['image']

    def test_noimages(self):
        images = buildImageDict(None, num=5)
        assert not images

    def test_realimages(self, release):
        imfiles = getImagesByPlate(plateid='8485', as_url=True, mode='local', release=release)
        images = buildImageDict(imfiles)
        names = [im['name'] for im in images]
        assert images
        assert '8485-1901' in names
        assert 'https://data.sdss.org/sas/mangawork/manga/spectro/redux/' in images[0]['image']




# !usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Licensed under a 3-clause BSD license.
#
# @Author: Brian Cherinka
# @Date:   2016-10-11 13:24:56
# @Last modified by:   Brian Cherinka
# @Last Modified time: 2018-06-30 14:01:05

from __future__ import absolute_import, division, print_function

import os
import sys
from collections import OrderedDict

import six


if ((sys.version_info.major == 3 and sys.version_info.minor > 2) or
        (sys.version_info.major == 2 and sys.version_info.minor >= 7)):
    from configparser import ConfigParser as SafeConfigParser
else:
    from configparser import SafeConfigParser


class Tree(object):
    ''' Initialize the sdss tree object

    This class provides Python programmatic access to the SDSS tree envionment structure

    Parameters:
        key (str|list):
            A section or list of sections of the tree to add into the local environment
        uproot_with (str):
            A new TREE_DIR path used to override an existing TREE_DIR environment variable
        config (str):
            Name of manual config file to load.  Default is sdsswork.
        update (bool):
            If True, overwrites existing tree environment variables in your
            local environment.  Default is False.
        exclude (list):
            A list of environment variables to exclude
            from forced updates

    Attributes:
        treedir (str):
            The directory of the tree
        environ (dict):
            The fully loaded SDSS config file held internally

    '''

    def __init__(self, *args, **kwargs):
        self.key = kwargs.get('key', None)
        uproot_with = kwargs.get('uproot_with', None)
        update = kwargs.get('update', False)
        self.exclude = kwargs.get('exclude', [])
        self.config_name = kwargs.get('config', 'sdsswork')
        self.set_roots(uproot_with=uproot_with)
        self.load_config()
        self.branch_out(limb=self.key)
        # add the general directories
        if self.key is not None:
            self.add_paths_to_os(key='general', update=update)
        self.add_paths_to_os(key=self.key, update=update)

    def __repr__(self):
        return ('Tree(sas_base_dir={0}, config={1})'.format(self.sasbasedir, self.config_name))

    def list_keys(self):
        ''' List the available keys you can load '''
        return [k for k in self.environ.keys() if k not in ['general', 'default']]

    def set_roots(self, uproot_with=None):
        ''' Set the roots of the tree in the os environment

        Parameters:
            uproot_with (str):
                A new TREE_DIR path used to override an existing TREE_DIR environment variable

        '''

        # Check for TREE_DIR
        self.treedir = os.environ.get('TREE_DIR', None) if not uproot_with else uproot_with
        if not self.treedir:
            treefilepath = os.path.dirname(os.path.abspath(__file__))
            if 'python/' in treefilepath:
                self.treedir = treefilepath.rsplit('/', 2)[0]
            else:
                self.treedir = treefilepath
            self.treedir = treefilepath
            os.environ['TREE_DIR'] = self.treedir

        # Check sas_base_dir
        if 'SAS_BASE_DIR' in os.environ:
            self.sasbasedir = os.environ["SAS_BASE_DIR"]
        else:
            self.sasbasedir = os.path.expanduser('~/sas')

        # make the directories
        if not os.path.isdir(self.sasbasedir):
            os.makedirs(self.sasbasedir)

    def load_config(self, config=None):
        ''' loads a config file

        Parameters:
            config (str):
                Optional name of manual config file to load
        '''

        # Read the config file
        cfgname = (config or self.config_name)
        cfgname = 'sdsswork' if cfgname is None else cfgname
        assert isinstance(cfgname, six.string_types), 'config name must be a string'
        config_name = cfgname if cfgname.endswith('.cfg') else '{0}.cfg'.format(cfgname)
        self.configfile = os.path.join(self.treedir, 'data', config_name)

        assert os.path.isfile(self.configfile) is True, 'configfile {0} must exist in the proper directory'.format(self.configfile)

        self._cfg = SafeConfigParser()

        try:
            self._cfg.read(self.configfile.decode('utf-8'))
        except AttributeError:
            self._cfg.read(self.configfile)

        # create the local tree environment
        self.environ = OrderedDict()
        self.environ['default'] = self._cfg.defaults()
        # set the filesystem envvar to sas_base_dir
        self._file_replace = '@FILESYSTEM@'
        if self.environ['default']['filesystem'] == self._file_replace:
            self.environ['default']['filesystem'] = self.sasbasedir

    def branch_out(self, limb=None):
        ''' Set the individual section branches

        This adds the various sections of the config file into the
        tree environment for access later. Optically can specify a specific
        branch.  This does not yet load them into the os environment.

        Parameters:
            limb (str/list):
                The name of the section of the config to add into the environ
                or a list of strings

        '''

        # Filter on sections
        if not limb:
            limbs = self._cfg.sections()
        else:
            # we must have the general always + secton
            limb = limb if isinstance(limb, list) else [limb]
            limbs = ['general']
            limbs.extend(limb)

        # add all limbs into the tree environ
        for leaf in limbs:
            leaf = leaf if leaf in self._cfg.sections() else leaf.upper()
            self.environ[leaf] = OrderedDict()
            options = self._cfg.options(leaf)

            for opt in options:
                if opt in self.environ['default']:
                    continue
                val = self._cfg.get(leaf, opt)
                if val.find(self._file_replace) == 0:
                    val = val.replace(self._file_replace, self.sasbasedir)
                self.environ[leaf][opt] = val

    def add_limbs(self, key=None):
        ''' Add a new section from the tree into the existing os environment

        Parameters:
            key (str):
                The section name to grab from the environment

        '''
        self.branch_out(limb=key)
        self.add_paths_to_os(key=key)

    def get_paths(self, key):
        ''' Retrieve a set of environment paths from the config

        Parameters:
            key (str):
                The section name to grab from the environment

        Returns:
            self.environ[newkey] (OrderedDict):
                An ordered dict containing all of the paths from the
                specified section, as key:val = name:path
        '''
        newkey = key if key in self.environ else key.upper() if key.upper() \
            in self.environ else None
        if newkey:
            return self.environ[newkey]
        else:
            raise KeyError('Key {0} not found in tree environment'.format(key))

    def add_paths_to_os(self, key=None, update=None):
        ''' Add the paths in tree environ into the os environ

        This code goes through the tree environ and checks
        for existence in the os environ, then adds them

        Parameters:
            key (str):
                The section name to check against / add
            update (bool):
                If True, overwrites existing tree environment variables in your
                local environment.  Default is False.
        '''

        if key is not None:
            allpaths = key if isinstance(key, list) else [key]
        else:
            allpaths = [k for k in self.environ.keys() if 'default' not in k]

        for key in allpaths:
            paths = self.get_paths(key)
            self.check_paths(paths, update=update)

    def check_paths(self, paths, update=None):
        ''' Check if the path is in the os environ, and if not add it

        Paramters:
            paths (OrderedDict):
                An ordered dict containing all of the paths from the
                a given section, as key:val = name:path
            update (bool):
                If True, overwrites existing tree environment variables in your
                local environment.  Default is False.
        '''

        # set up the exclusion list
        exclude = [] if not self.exclude else self.exclude \
            if isinstance(self.exclude, list) else [self.exclude]

        # check the path names
        for pathname, path in paths.items():
            if update and pathname.upper() not in exclude:
                os.environ[pathname.upper()] = os.path.normpath(path)
            elif pathname.upper() not in os.environ:
                os.environ[pathname.upper()] = os.path.normpath(path)

    def replant_tree(self, config=None, exclude=None):
        ''' Replant the tree with a different config setup

        Parameters:
            config (str):
                The config name to reload
            exclude (list):
                A list of environment variables to exclude
                from forced updates
        '''

        # reinitialize a new Tree with a new config
        self.__init__(key=self.key, config=config, update=True, exclude=exclude)



# !usr/bin/env python
# -*- coding: utf-8 -*-
#
# Licensed under a 3-clause BSD license.
#
# @Author: Brian Cherinka
# @Date:   2018-11-20 16:32:15
# @Last modified by:   Brian Cherinka
# @Last Modified time: 2018-11-29 18:46:25

from __future__ import print_function, division, absolute_import

from brain import bconfig
from brain.api.api import BrainInteraction, BrainAuth
import pytest
import requests

auths = [None, 'token', 'netrc']


@pytest.fixture(params=auths)
def brainint(request):
    base = 'https://lore.sdss.utah.edu/test/'
    url = '/marvin/api/general/getroutemap/'
    ii = BrainInteraction(url, auth=request.param, send=False, base=base)
    yield ii
    ii = None


@pytest.fixture(scope='session')
def req():
    r = requests.get('https://data.sdss.org/sas/mangawork/manga/spectro/redux/v2_4_3/8485/stack/images/1901.png')
    yield r
    r = None


class TestBrainAuth(object):

    @pytest.mark.parametrize('auth', [(None), ('token'), ('netrc')])
    def test_init(self, auth):
        a = BrainAuth(authtype=auth)
        assert a.authtype == auth

    @pytest.mark.parametrize('auth, check',
                             [(None, None),
                              ('token', 'Bearer testtoken'),
                              ('netrc', 'Basic')], ids=['none', 'token', 'netrc'])
    def test_call(self, monkeypatch, req, auth, check):
        if auth == 'token':
            monkeypatch.setattr(bconfig, 'token', 'testtoken')

        a = BrainAuth(authtype=auth)
        resp = a(req)

        if not auth:
            assert 'Authorization' not in resp.headers
        else:
            assert 'Authorization' in resp.headers
            assert check in resp.headers['Authorization']


    def test_token_fail(self):
        a = BrainAuth(authtype='token')

        with pytest.raises(AssertionError) as cm:
            r = a(req)
        assert 'You must have a valid token set to use the API.  Please login.' in str(cm.value)


class TestInteraction(object):

    def test_auth(self, brainint):
        assert brainint.authtype in auths
        if brainint.authtype is None:
            assert brainint.session.auth is None
        else:
            assert brainint.authtype == brainint.session.auth.authtype




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# @Author: Brian Cherinka, José Sánchez-Gallego, and Brett Andrews
# @Date: 2018-07-24
# @Filename: test_rss.py
# @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)
#
# @Last modified by: José Sánchez-Gallego (gallegoj@uw.edu)
# @Last modified time: 2018-08-04 13:35:39

import astropy.io.fits
import astropy.table
import numpy
import pytest

import marvin

from ..conftest import Galaxy, set_the_config


@pytest.fixture(scope='session')
def galaxy(get_params, plateifu):
    """Yield an instance of a Galaxy object for use in tests."""

    release, bintype, template = get_params

    set_the_config(release)

    gal = Galaxy(plateifu=plateifu)
    gal.set_params(bintype=bintype, template=template, release=release)
    gal.set_filepaths()
    gal.set_galaxy_data()

    yield gal


@pytest.fixture(scope='session')
def rss_session(galaxy, mode):
    # These get created only once per session.

    if mode == 'auto' or str(galaxy.bintype) != 'SPX':
        pytest.skip()

    if mode == 'local':
        rss = marvin.tools.RSS(filename=galaxy.rsspath, release=galaxy.release, mode='local')
    else:
        rss = marvin.tools.RSS(plateifu=galaxy.plateifu, release=galaxy.release, mode='remote')

    yield rss


@pytest.fixture(scope='function')
def rss(rss_session):

    # In some of the tests we modify the RSS objects. Here we implement
    # a setup procedure that "unloads" the RSSFiber objects and resets the
    # autoload attribute.

    for rssfiber in rss_session:
        rssfiber.loaded = False

    rss_session.autoload = True

    yield rss_session


@pytest.fixture(scope='session')
def rssfiber(rss_session):

    fiberid = 0

    if rss_session[fiberid].loaded is False:
        rss_session[fiberid].load()

    yield rss_session[fiberid]


@pytest.mark.usefixtures('monkeyauth')
class TestRSS(object):

    def test_rss_init(self, rss):

        assert isinstance(rss, marvin.tools.RSS)
        assert isinstance(rss, marvin.tools.mixins.NSAMixIn)
        assert isinstance(rss, list)

        assert isinstance(rss.obsinfo, astropy.table.Table)

        if rss.mode == 'file':
            assert isinstance(rss.data, astropy.io.fits.HDUList)

        assert rss._wavelength is not None
        assert len(rss) == rss._nfibers

        rss.autoload = False  # To make things faster for this test
        assert all([isinstance(rss_fiber, marvin.tools.rss.RSSFiber) for rss_fiber in rss])

    @pytest.mark.parametrize('autoload', [True, False])
    def test_rss_autoload(self, rss, autoload):

        rss.autoload = autoload
        assert rss[0].loaded is autoload

    def test_load(self, rss):

        rss.autoload = False
        assert rss[0].loaded is False

        rss[0].load()
        assert rss[0].loaded is True

    def test_load_all(self, rss):

        if rss.mode == 'remote':
            pytest.skip()

        rss.load_all()
        assert all([rss_fiber.loaded is True for rss_fiber in rss])

    def test_obsinfo_to_rssfiber(self, rss):

        # We get it in this complicated way so that it is a different way of
        # obtianing it than in the _populate_fibres method.
        ifusize = int(str(rss.ifu)[0:-2])

        exp_idx = 0
        n_fiber = 1
        for rssfiber in rss:

            assert numpy.all(rss.obsinfo[exp_idx] == rssfiber.obsinfo)

            n_fiber += 1
            if n_fiber > ifusize:
                n_fiber = 1
                exp_idx += 1

    def test_getcube(self, rss):

        cube = rss.getCube()

        assert isinstance(cube, marvin.tools.Cube)
        assert cube.mode == rss.mode
        assert cube.plateifu == rss.plateifu
        assert cube.mangaid == rss.mangaid
        assert cube.release == rss.release

    def test_select_fibers(self, rss):

        # Skipping for API or it will take forever. Should not matter since
        # we have already tested slicing for API.
        if rss.data_origin == 'api':
            pytest.skip()

        fibers_expnum = rss.select_fibers(exposure_no=198575)
        assert len(fibers_expnum) == 19
        assert fibers_expnum[0].obsinfo['EXPNUM'][0] == 198575

        fibers_mjd = rss.select_fibers(mjd=1234)
        assert len(fibers_mjd) == 0

        fibers_mjd = rss.select_fibers(mjd=57132)
        assert len(fibers_mjd) == (9 * 19)
        assert fibers_mjd[0].obsinfo['MJD'][0] == 57132



@pytest.mark.usefixtures('monkeyauth')
class TestRSSFiber(object):

    def test_rssfiber_spectra(self, rssfiber):

        assert isinstance(rssfiber, marvin.tools.RSSFiber)
        assert isinstance(rssfiber.rss, marvin.tools.RSS)

        assert isinstance(rssfiber.obsinfo, astropy.table.Table)

        assert hasattr(rssfiber, 'ivar')
        assert isinstance(rssfiber.ivar, numpy.ndarray)
        assert len(rssfiber.ivar) == len(rssfiber.wavelength)

        assert hasattr(rssfiber, 'mask')
        assert isinstance(rssfiber.mask, numpy.ndarray)
        assert len(rssfiber.mask) == len(rssfiber.wavelength)

        for dm_element in rssfiber.rss.datamodel.rss + rssfiber.rss.datamodel.spectra:

            if dm_element.name == 'flux':
                continue

            spectrum = getattr(rssfiber, dm_element.name, None)
            assert spectrum is not None
            assert isinstance(spectrum, numpy.ndarray)
            assert len(spectrum) == len(rssfiber.wavelength)

    def test_rssfiber_data(self, rssfiber):

        rss_filename = rssfiber.rss._getFullPath()
        rss_hdu = astropy.io.fits.open(rss_filename)

        numpy.testing.assert_allclose(rss_hdu['FLUX'].data[rssfiber.fiberid, :], rssfiber.value)
        numpy.testing.assert_allclose(rss_hdu['IVAR'].data[rssfiber.fiberid, :], rssfiber.ivar)
        numpy.testing.assert_array_equal(rss_hdu['MASK'].data[rssfiber.fiberid, :], rssfiber.mask)

        for dm_element in rssfiber.rss.datamodel.rss:
            if dm_element.name == 'flux':
                continue
            fits_data = rss_hdu[dm_element.fits_extension()].data[rssfiber.fiberid, :]
            numpy.testing.assert_allclose(fits_data, getattr(rssfiber, dm_element.name).value)

        for dm_element in rssfiber.rss.datamodel.spectra:
            fits_data = rss_hdu[dm_element.fits_extension()].data
            numpy.testing.assert_allclose(fits_data, getattr(rssfiber, dm_element.name).value)

    def test_rssfiber_slice(self, rssfiber):

        n_elements = 10

        sliced = rssfiber[0:n_elements]

        assert len(sliced.value) == n_elements
        numpy.testing.assert_allclose(sliced.value, rssfiber.value[0:n_elements])

        assert len(sliced.ivar) == n_elements
        assert len(sliced.mask) == n_elements

        for dm_element in rssfiber.rss.datamodel.rss + rssfiber.rss.datamodel.spectra:

            if dm_element.name == 'flux':
                continue

            spectrum_sliced = getattr(sliced, dm_element.name, None)
            assert len(spectrum_sliced) == n_elements

        assert sliced.obsinfo is not None

    def test_rssfiber_masked(self, rssfiber):

        assert numpy.sum(rssfiber.masked.mask) > 0

    def test_rssfiber_descale(self, rssfiber):

        descaled = rssfiber.descale()
        numpy.testing.assert_allclose(descaled.value, rssfiber.value * rssfiber.unit.scale)

        assert descaled.obsinfo is not None


class TestPickling(object):

    def test_pickling_file(self, temp_scratch, rss):

        if rss.data_origin == 'file':
            assert rss.data is not None

        rss_file = temp_scratch.join('test_rss.mpf')
        rss.save(str(rss_file))
        assert rss_file.check() is True

        rss_restored = marvin.tools.RSS.restore(str(rss_file))

        assert rss_restored.data_origin == rss.data_origin
        assert isinstance(rss_restored, marvin.tools.RSS)

        assert len(rss_restored) > 0
        assert isinstance(rss_restored[0], marvin.tools.RSSFiber)
        assert numpy.sum(rss_restored[0].value) > 0

        if rss.data_origin == 'file':
            assert rss_restored.data is not None
        else:
            assert rss_restored.data is None
